@@ -2,12 +2,12 @@
 #
 #       Module:  Term::CLI::ArgumentSet
 #
-#  Description:  Class for sets of (sub-)commands in Term::CLI
+#  Description:  Role for sets of arguments in Term::CLI
 #
 #       Author:  Steven Bakker (SBAKKER), <sbakker@cpan.org>
 #      Created:  05/02/18
 #
-#   Copyright (c) 2018 Steven Bakker
+#   Copyright (c) 2018-2022 Steven Bakker
 #
 #   This module is free software; you can redistribute it and/or modify
 #   it under the same terms as Perl itself. See "perldoc perlartistic."
@@ -18,7 +18,7 @@
 #
 #=============================================================================
 
-package Term::CLI::Role::ArgumentSet 0.054002;
+package Term::CLI::Role::ArgumentSet 0.055001;
 
 use 5.014;
 use warnings;
@@ -88,11 +88,11 @@ __END__
 
 =head1 NAME
 
-Term::CLI::Role::ArgumentSet - Role for (sub-)commands in Term::CLI
+Term::CLI::Role::ArgumentSet - Manage command arguments in Term::CLI
 
 =head1 VERSION
 
-version 0.054002
+version 0.055001
 
 =head1 SYNOPSIS
 
@@ -178,31 +178,6 @@ X<argument_names>
 
 Return the list of (sub-)command names (in the order they were specified).
 
-=item B<find_matches> ( I<Str> )
-X<find_matches>
-
-Return a list of all commands in this object that match the I<Str>
-prefix.
-
-=item B<find_command> ( I<Str> )
-X<find_command>
-
-Check whether I<Str> uniquely matches a command in this C<Term::CLI>
-object. Returns a reference to the appropriate
-L<Term::CLI::Command> object if successful; otherwise, it
-sets the objects C<error> field and returns C<undef>.
-
-Example:
-
-    my $sub_cmd = $cmd->find_command($prefix);
-    die $cmd->error unless $sub_cmd;
-
-=item B<try_callback> ( I<ARGS> )
-X<try_callback>
-
-Wrapper function that will call the object's C<callback> function if it
-has been set, otherwise simply returns its arguments.
-
 =back
 
 =head1 SEE ALSO
