@@ -260,7 +260,7 @@ push @commands, Term::CLI::Command->new(
                 Term::CLI::Argument::Bool->new(
                     name         => 'bool',
                     true_values  => [qw( 1 true on yes ok )],
-                    false_values => [qw( 1 false off no never )],
+                    false_values => [qw( 0 false off no never )],
                 )
 
             ],
@@ -367,6 +367,7 @@ push @commands, Term::CLI::Command->new(
         . "signal and return control to the shell.",
     callback => sub {
         my ( $cmd, %args ) = @_;
+        return %args if $args{status} < 0;
         local ( $SIG{TSTP} ) = 'DEFAULT';
         kill 'TSTP', $$;
         return %args;
@@ -379,7 +380,7 @@ $term->read_history();
 
 say "\n[Welcome to BSSH]";
 while ( defined( my $line = $term->readline ) ) {
-    $term->execute($line);
+    $term->execute_line($line);
 }
 print "\n";
 execute_exit( $term, 0 );
